#include "cppdefs.h"
!-----------------------------------------------------------------------
!BOP
!
! !MODULE: ice --- atmospheric fluxes \label{sec:ice}
!
! !INTERFACE:
   module ice
!
! !DESCRIPTION:
!  To be done
!
! !USES:
   use time,             only: julianday, secondsofday
   use airsea_variables, only: emiss,bolz,kelvin
   use ice_winton,       only: do_ice_winton, ice_optics, KMELT, CW
   use ice_uvic,         only: init_ice_uvic, do_ice_uvic
   use ice_uvic,         only: nilay,nlmax,Tfreezi
   use meanflow,         only: h,T,S,rho,rho_0
   use airsea,           only: heat,I_0,albedo,precip,evap,cloud,swr_method,airt, &
                               airp,rh,u10,v10,back_radiation_method,hum_method, &
                               fluxes_method
!
   IMPLICIT NONE

!  default: all is private.
   private
!
! !PUBLIC MEMBER FUNCTIONS:
   public                              :: init_ice
   public                              :: do_ice
   public                              :: clean_ice
#ifdef _PRINTSTATE_
   public                              :: print_state_ice
#endif
!
! !PUBLIC DATA MEMBERS:
   integer, public                     :: ice_method
!  Simple 'ice model'
   REALTYPE, public                    :: ice_layer
!  Winton/UVIC ice model
   REALTYPE, public, target            :: ice_hs,ice_hi
!  Winton ice model
   REALTYPE, public                    :: ice_ts,ice_T1,ice_T2
   REALTYPE, public                    :: ice_tmelt,ice_bmelt
!  University of Victoria ice model
! ts,tb are the upper surface temperature of the ice/snow and tb is the lowest layer temperature (not the bottom temperature, which is set to Tfreezi) all in (K)
   REALTYPE, public, target            :: ice_uvic_ts, ice_uvic_tb
   REALTYPE, public, target            :: ice_uvic_swr_0,ice_uvic_precip_i,ice_uvic_sfall_i
   REALTYPE, public, target            :: ice_uvic_parb, ice_uvic_parui
   REALTYPE, public, target            :: ice_uvic_topmelt,ice_uvic_botmelt
   REALTYPE, public, target            :: ice_uvic_topgrowth,ice_uvic_botgrowth
   REALTYPE, public, target            :: ice_uvic_termelt,ice_uvic_Fh
   REALTYPE, public                    :: ice_uvic_Ff,ice_uvic_Fs
   REALTYPE, public                    :: ice_uvic_Sicebulk
   REALTYPE, public                    :: ice_uvic_Hmix
   REALTYPE, public                    :: ice_uvic_Aice,ice_uvic_Asnow
   REALTYPE, public, target            :: ice_uvic_Amelt
   REALTYPE, public                    :: ice_uvic_hm
!  University of Victoria ice model: allocatable variables
!  ice_uvic_Tice  - temperature array (dimensioned nlay+1), Tice(1) is the
!                 upper surface temperature, Tice(nlay+1) is the bottom
!                 temperature, remaining values at layer interfaces (K)
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_Tice
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_Cond
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_rhoCp
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_Sint
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_dzi
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_zi
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_Told
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_Pari
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_dum
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_dzice
   REALTYPE, public, dimension(:), allocatable           :: ice_uvic_zice
!
!  !PRIVATE DATA MEMBERS:
   REALTYPE                            :: lat, lon

! !REVISION HISTORY:
!  Original author(s): Karsten Bolding
!
! !LOCAL VARIABLES:
!EOP
!-----------------------------------------------------------------------

   contains

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: Initialise the ice module \label{sec:init-ice}
!
! !INTERFACE:
   subroutine init_ice(namlst,latitude,longitude)
!
! !DESCRIPTION:
!  To be done
!
! !USES:
   IMPLICIT NONE
!
! !INPUT PARAMETERS:
   integer, intent(in)                 :: namlst
   REALTYPE, intent(in)                :: latitude,longitude

!
! !REVISION HISTORY:
!  Original author(s): Karsten Bolding
!
! !LOCAL VARIABLES:
   namelist /ice/ ice_method
   integer :: k,rc
!EOP
!-----------------------------------------------------------------------
!BOC
   LEVEL1 'init_ice'
   lat = latitude
   lon = longitude

!  initialize namelist variables to reasonable defaults.
   ice_method=0

!  Read namelist variables from file.
   open(namlst,file='ice.nml',action='read',status='old',err=90)
   read(namlst,nml=ice,err=91)
   close(namlst)

!  The different ice models
   select case (ice_method)
      case (0)
         LEVEL2 'No ice calculations included'
      case (1)
         LEVEL2 'Clip heat-fluxes if SST < freezing point (function of S)'
         ice_layer=_ZERO_
      case (2)
         LEVEL2 'Thermodynamic ice model adapted from Winton'
         ice_hs=_ZERO_;ice_hi=_ZERO_;ice_T1=_ZERO_;ice_T2=_ZERO_
         ice_ts=_ZERO_;ice_tmelt=_ZERO_;ice_bmelt=_ZERO_
      case (3)
         LEVEL2 'Thermodynamic ice model adapted from Flato&Brown, 1992, UVic'
         call init_ice_uvic(namlst)
         allocate(ice_uvic_Tice(nilay+1),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_Tice)'
         ice_uvic_Tice =_ZERO_
            do k=1,nilay+1
               ice_uvic_Tice(k)=245.+(Tfreezi-245.)*float(k-1)/float(nilay)
            enddo
         allocate(ice_uvic_Cond(nilay),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_Cond)'
         ice_uvic_Cond =_ZERO_
         allocate(ice_uvic_rhoCp(nilay),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_rhoCp)'
         ice_uvic_rhoCp =_ZERO_
         allocate(ice_uvic_Sint(nilay+1),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_Sint)'
         ice_uvic_Sint =_ZERO_
         allocate(ice_uvic_dzi(nilay),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_dzi)'
         ice_uvic_dzi =_ZERO_
         allocate(ice_uvic_zi(nilay+1),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_zi)'
         ice_uvic_zi =_ZERO_
         allocate(ice_uvic_Told(nilay+1),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_Told)'
         ice_uvic_Told =_ZERO_
         allocate(ice_uvic_Pari(nilay+1),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_Pari)'
         ice_uvic_Pari =_ZERO_
         allocate(ice_uvic_dum(nilay+1),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_dum)'
         allocate(ice_uvic_dzice(nilay),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_dzi)'
            do k=1,nilay
               ice_uvic_dzice(k)=float(k)
            enddo
         allocate(ice_uvic_zice(nilay+1),stat=rc)
         if (rc /= 0) STOP 'init_ice: Error allocating (ice_uvic_zice)'
            do k=1,nilay+1
               ice_uvic_zice(k)=float(k)
            enddo

         ice_uvic_dum =_ZERO_
         ice_hs=_ZERO_;ice_hi=_ZERO_;ice_uvic_hm=_ZERO_
         ice_uvic_ts=273.16D+00;ice_uvic_tb=273.16D+00;ice_uvic_Fh=_ZERO_
         ice_uvic_swr_0=_ZERO_;ice_uvic_precip_i=_ZERO_;ice_uvic_sfall_i=_ZERO_
         ice_uvic_parb=_ZERO_;ice_uvic_parui=_ZERO_;
         ice_uvic_Ff=_ZERO_;ice_uvic_Fs=_ZERO_
         ice_uvic_Sicebulk=6.0D+00
         ice_uvic_topmelt=_ZERO_;ice_uvic_botmelt=_ZERO_
         ice_uvic_termelt=_ZERO_;ice_uvic_topgrowth=_ZERO_
         ice_uvic_botgrowth=_ZERO_;ice_uvic_Hmix=_ZERO_ 
         ice_uvic_Aice=_ZERO_;ice_uvic_Asnow=_ZERO_ 
         ice_uvic_Amelt=_ZERO_ 

      case default
   end select

   return

90 FATAL 'I could not open ice.nml'
   stop 'init_ice'
91 FATAL 'I could not read ice namelist'
   stop 'init_ice'

   end subroutine init_ice
!EOC

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: Obtain the air--sea fluxes
!
! !INTERFACE:
   subroutine do_ice(dt)
!
! !DESCRIPTION:
!  To be done
!
! !USES:
   IMPLICIT NONE
! !INPUT PARAMETERS:
   REALTYPE, intent(in)                :: dt
!
! !REVISION HISTORY:
!  Original author(s): Karsten Bolding, Jesper Larsen
!
! !LOCAL VARIABLES:
   REALTYPE        :: tfw
   integer         :: n
   logical         :: has_ice
!EOP
!-----------------------------------------------------------------------
!BOC

!  The different ice models
   select case (ice_method)
      case (0)
         LEVEL2 'ice_method=',ice_method
      case (1)
!         LEVEL0 T(10),S(10),heat
         n = ubound(S,1)
         tfw = -0.0575*S(n)
         if (heat .gt. _ZERO_ .and. T(n) .le. tfw) then
            heat = _ZERO_
            LEVEL0 'do_ice: heat clipped to',heat
            ice_layer=_ONE_
         else
            ice_layer=_ZERO_
         end if
      case (2)
         if (swr_method .ne. 3) then
           ! STDERR 'Ice model currently only supports swr_method 3'
           ! stop 'Ice model currently only supports swr_method 3'
         endif
         n = ubound(S,1)
         call do_ice_winton(julianday,secondsofday,lon,lat, &
                            cloud,airt,airp,rh,u10,v10, &
                            S(n),rho(n),rho_0,h(n), &
                            back_radiation_method,hum_method, &
                            fluxes_method,dt, &
                            T(n),heat,I_0,precip, &
                            ice_hs,ice_hi,ice_t1,ice_t2, &
                            ice_ts,albedo,ice_tmelt,ice_bmelt)
      case (3)
         if (swr_method .ne. 3) then
            !STDERR 'Ice model currently only support swr_method 3'
            !stop 'Ice model currently only support swr_method 3'
         endif
         n = ubound(S,1)
         call do_ice_uvic(dt,h(n),julianday,secondsofday,lon,lat, &
                          I_0,airt,airp,rh,u10,v10,precip,cloud, &
                          T(n),S(n),rho(n),rho_0, &
                          back_radiation_method,hum_method,fluxes_method,&
                          ice_hi,ice_hs,ice_uvic_hm,ice_uvic_Tice, &
                          ice_uvic_Cond,ice_uvic_rhoCp, &
                          ice_uvic_Sint,ice_uvic_dzi,ice_uvic_zi, &
                          ice_uvic_Pari,ice_uvic_Told,albedo,heat,&
                          ice_uvic_Fh,ice_uvic_Ff,ice_uvic_Fs,&
                          ice_uvic_Sicebulk, ice_uvic_topmelt, &
                          ice_uvic_botmelt,ice_uvic_termelt, &
                          ice_uvic_topgrowth,ice_uvic_botgrowth,&
                          ice_uvic_Hmix,ice_uvic_Aice,ice_uvic_Asnow,&
                          ice_uvic_Amelt,ice_uvic_swr_0,ice_uvic_precip_i,ice_uvic_sfall_i)
                   
                    ice_uvic_ts=ice_uvic_Tice(1)
                    ice_uvic_tb=ice_uvic_Tice(nilay)
                    ice_uvic_parb=ice_uvic_Pari(nilay)
                    ice_uvic_parui=ice_uvic_Pari(nilay+1)
  

      case default
   end select      

   end subroutine do_ice
!EOC

!-----------------------------------------------------------------------
!BOP
!
! !IROUTINE: Finish the air--sea interactions
!
! !INTERFACE:
   subroutine clean_ice
!
! !DESCRIPTION:
!  All files related to air-sea interaction which have been opened
!  are now closed by this routine.
!
! !USES:
   IMPLICIT NONE
!
! !REVISION HISTORY:
!  Original author(s): Karsten Bolding
!
!EOP
!-----------------------------------------------------------------------
!BOC
!  The different ice models
   select case (ice_method)
      case (0)
         LEVEL2 'No ice cleaning'
      case (1)
         LEVEL2 'No ice cleaning'
      case (2)
         LEVEL2 'No ice cleaning'
      case (3)
         LEVEL2 'ice_uvic: de-allocation icemodel memory ...'
         if (allocated(ice_uvic_Tice)) deallocate(ice_uvic_Tice)
         if (allocated(ice_uvic_Cond)) deallocate(ice_uvic_Cond)
         if (allocated(ice_uvic_rhoCp)) deallocate(ice_uvic_rhoCp)
         if (allocated(ice_uvic_Sint)) deallocate(ice_uvic_Sint)
         if (allocated(ice_uvic_dzi)) deallocate(ice_uvic_dzi)
         if (allocated(ice_uvic_zi)) deallocate(ice_uvic_zi)
         if (allocated(ice_uvic_Told)) deallocate(ice_uvic_Told)
         if (allocated(ice_uvic_Pari)) deallocate(ice_uvic_Pari)
         if (allocated(ice_uvic_dum)) deallocate(ice_uvic_dum)

      case default
   end select

   end subroutine clean_ice
!EOC

   end module ice

!-----------------------------------------------------------------------
! Copyright by the GOTM-team under the GNU Public License - www.gnu.org
!-----------------------------------------------------------------------
